#brailleDisplayDrivers/alvaSatellite.py
#A part of NonVisual Desktop Access (NVDA)
#Copyright (C) 2006-2009 NVDA Contributors <http://www.nvda-project.org/>
#This file is covered by the GNU General Public License.
#See the file COPYING for more details.

import os
import braille
import queueHandler
from logHandler import log
from ctypes import *
import time
import wx
import config

ALVA_KEY_CHECK_INTERVAL = 50

# Key groups
KEY_FLAGS = 0x70
KEY_OK = 0x71
KEY_TCS = 0x71
KEY_DTCS = 0x71
KEY_TC = 0x72
KEY_ERROR = 0xFF

# Try to load alvaw32.dll
try:
	AlvaLib = windll[os.path.abspath(os.path.join(os.path.dirname(__file__), r"abtusb32.dll"))]
	AlvaLib.ABT_GetABTType.restype = c_byte*4
	AlvaLib.ABT_GetKeyStatus.restype = c_byte*4
except:
	AlvaLib=None

class BrailleDisplayDriver(braille.BrailleDisplayDriver):
	name = "alvaSatellite"
	description = _("ALVA Satellite 544/574 series")

	@classmethod
	def check(cls):
		if AlvaLib:
			log.info("Alva library was loaded")
		else:
			log.info("Alva library was not loaded")

		return bool(AlvaLib)

	def __init__(self):
		super(BrailleDisplayDriver,self).__init__()
		log.debug("ALVA Satellite Braille init")
		if not AlvaLib.ABT_InstallABTport("COM1"):
			raise RuntimeError("No ALVA display found")
		type = AlvaLib.ABT_GetABTType()
		log.info("ALVA Satellite %d%d device found, %d cells, %d status cells" % (type[1], type[0], type[2], type[3]))
		AlvaLib.ABT_Clear()
		self._alva_NumCells = type[2]
		self._pressed_keys= []
		self._alva_KeyCheckTimer = wx.PyTimer(self._alva_CheckKeyPresses)
		self._alva_KeyMask = 0
		self._alva_KeyCheckTimer.Start(ALVA_KEY_CHECK_INTERVAL)

	def terminate(self):
		super(BrailleDisplayDriver, self).terminate()
		AlvaLib.ABT_Clear()
		try:
			self._alva_KeyCheckTimer.Stop()
			self._alva_KeyCheckTimer = None
		except:
			pass

	def _get_numCells(self):
		return self._alva_NumCells

	def _display(self, cells):
		#log.info("Display %d cells" %len(cells))
		cells="".join([chr(x) for x in cells])
		AlvaLib.ABT_BrDisplay(cells, 0, len(cells))

	def _alva_CheckKeyPresses(self):
		while True:
			try:
				key_status = AlvaLib.ABT_GetKeyStatus()
				group = key_status[1]
				key_code = key_status[2]
			except:
				log.error("Error reading keypress from ABTUSB32.DLL", exc_info=True)
				return
			if group == KEY_FLAGS:
				break
			is_key_release = True
			if group == KEY_OK and key_code <= 9:
				is_key_release = False
			
			if is_key_release:
				self._alva_OnKeyRelease()
			else:
				self._alva_OnKeyPress(group, key_code)

	def _alva_OnKeyPress(self, group, key_code):
		key_tuple = (group, key_code)
		if not key_tuple in self._pressed_keys:
			self._pressed_keys.append(key_tuple)
		#braille.handler.routeTo(KeyNumber)

	def _alva_OnKeyRelease(self):
		pressed_keys = self._pressed_keys
		self._pressed_keys = []
		braille.handler.scrollBack()